'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.createPage = undefined;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

var _updateNotifier = require('update-notifier');

var _updateNotifier2 = _interopRequireDefault(_updateNotifier);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const cwd = process.cwd();

const defaultValue = {
	name: 'index',
	indent: 'tab',
	dir: cwd,
	json: false,
	style: 'wxss'
};

const formatTemplate = (template, pathname, basename, indent) => {
	let templateStr = template({ pathname, basename });
	if (indent !== defaultValue.indent) {
		const intNum = ~~indent;
		const spaces = new Array(intNum).fill(' ').join('');
		templateStr = templateStr.replace(/\t/g, spaces);
	}
	return templateStr;
};

const parseName = name => {
	name = name.split(/[\\\/]/);
	const len = name.length;
	if (len === 1) name.push(name[0]);
	if (!name[len - 1]) name[len - 1] = name[len - 2];
	if (name[0]) name.unshift('pages');else name.shift();

	return {
		basename: name.pop(),
		pathname: name.join('/')
	};
};

const getRootDir = dir => (0, _path.resolve)(cwd, dir);

const validateAppJson = rootDir => {
	const exists = _fs2.default.existsSync((0, _path.resolve)(rootDir, 'app.json'));
	if (!exists) {
		throw new Error(`${ rootDir } 目录找不到 app.json 文件！`);
	}
	return true;
};

const addRoute = (rootDir, pathname, basename, indent) => {
	const filename = rootDir + '/app.json';
	const content = _fs2.default.readFileSync(filename, 'utf8');
	const json = JSON.parse(content);
	const newPath = [pathname, basename].filter(Boolean).join('/');
	if (json.pages.indexOf(newPath) < 0) {
		json.pages.push(newPath);
	}
	const formatedIndent = indent === 'tab' ? '\t' : ~~indent;
	const result = JSON.stringify(json, null, formatedIndent);
	_fs2.default.writeFileSync(filename, result);
};

const createPage = exports.createPage = options => {
	if (typeof options !== 'object') {
		throw new Error('参数必须是一个JSON对象。');
	}

	options = Object.assign({}, defaultValue, options);

	var _options = options;
	const dir = _options.dir,
	      name = _options.name,
	      indent = _options.indent,
	      json = _options.json,
	      style = _options.style;

	const rootDir = getRootDir(dir);

	var _parseName = parseName(name);

	const pathname = _parseName.pathname,
	      basename = _parseName.basename;


	validateAppJson(rootDir);

	const pageRoot = (0, _path.resolve)(rootDir, pathname);

	if (!_fs2.default.existsSync(pageRoot)) {
		_mkdirp2.default.sync(pageRoot);
	}

	const filesTypes = ['wxml', 'js', style];
	if (json) {
		filesTypes.push('json');
	}

	filesTypes.forEach(fileType => {
		fileType = fileType.replace(/^\./g, '');

		const filePath = (0, _path.resolve)(pageRoot, `${ basename }.${ fileType }`);

		if (_fs2.default.existsSync(filePath)) {
			throw new Error(`${ filePath } 已存在！`);
		}

		const notStyleFiles = ['js', 'wxml', 'json'];
		const templateExt = notStyleFiles.includes(fileType) ? fileType : 'wxss';

		var _require = require(`./templates/${ templateExt }`);

		const template = _require.default;


		const content = formatTemplate(template, pathname, basename, indent);
		_fs2.default.writeFileSync(filePath, content);
		console.log(`${ filePath } 创建成功；`);
	});

	addRoute(rootDir, pathname, basename, indent);
	console.log('创建结束。');
};

const app = (() => {
	var _ref = _asyncToGenerator(function* () {
		// eslint-disable-next-line
		const argv = _yargs2.default.options({
			i: {
				alias: 'indent',
				describe: '文件缩进的方式'
			},
			d: {
				alias: 'dir',
				describe: '小程序源代码根目录 (app.json 所在目录)',
				type: 'string'
			},
			n: {
				alias: 'name',
				describe: '生成页面的名称 (可包含路径)',
				type: 'string'
			},
			j: {
				alias: 'json',
				describe: '是否需要生成配置文件 (.json)',
				type: 'boolean'
			},
			s: {
				alias: 'style',
				describe: '样式文件的类型 (如 wcss, scss 等)',
				type: 'string'
			},
			y: {
				alias: 'yes',
				describe: '使用默认值生成相关文件',
				type: 'boolean'
			}
		}).alias('h', 'help').alias('v', 'version').help().version(_package.version).argv;

		const createPromptItems = function (options) {
			return [{
				message: '请输入小程序源代码根目录 (app.json 所在目录)',
				default: defaultValue.dir,
				type: 'input',
				name: 'dir',
				when: !options.dir,
				validate: function (answer) {
					const rootDir = getRootDir(answer);
					try {
						return validateAppJson(rootDir);
					} catch (err) {
						return err.message;
					}
				}
			}, {
				message: '请输入页面名称 (可包含路径)',
				default: defaultValue.name,
				type: 'input',
				name: 'name',
				when: !options.name
			}, {
				message: '请输入文件缩进的方式',
				default: defaultValue.indent,
				type: 'input',
				name: 'indent',
				when: !options.indent
			}, {
				message: '是否需要生成配置文件 (.json)',
				default: defaultValue.json,
				type: 'confirm',
				name: 'json',
				when: !options.json
			}, {
				type: 'list',
				message: '请选择样式文件的类型',
				default: defaultValue.style,
				choices: ['wxss', 'scss', 'less', 'pcss', 'styl', '自定义'],
				name: 'style',
				when: !options.style
			}, {
				message: '请输入自定义样式文件的类型',
				type: 'input',
				default: defaultValue.style,
				name: 'style',
				when: function (answers) {
					return !options.style && answers.style === '自定义';
				}
			}];
		};

		if (argv.yes) {
			Object.assign({}, defaultValue, argv);
		} else {
			const promptItems = createPromptItems(argv);
			const answers = yield _inquirer2.default.prompt(promptItems);
			Object.keys(answers).forEach(function (key) {
				argv[key] = answers[key];
			});
		}
		createPage(argv);
	});

	return function app() {
		return _ref.apply(this, arguments);
	};
})();

app().catch(err => console.error('[错误]', err.message));
(0, _updateNotifier2.default)({ pkg: _package2.default }).notify();